package cn.estsh.i3plus.pojo.mes.bean;

import cn.estsh.i3plus.pojo.base.bean.BaseBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiParam;
import lombok.Data;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import lombok.EqualsAndHashCode;
import org.hibernate.annotations.ColumnDefault;
import org.hibernate.annotations.DynamicInsert;
import org.hibernate.annotations.DynamicUpdate;

import javax.persistence.*;
import java.io.Serializable;

/**
 * @Description : 产品条码履历实体类
 * @Reference :
 * @Author : dragon.xu
 * @CreateDate : 2019-05-29 16:22
 * @Modify:
 **/
@Data
@Entity
@DynamicInsert
@DynamicUpdate
@EqualsAndHashCode(callSuper = true)
@Inheritance(strategy = InheritanceType.JOINED)
@Table(name = "MES_PRODUCE_SN_TRAVEL", indexes = {
        @Index(columnList = "SERIAL_NUMBER"),
        @Index(columnList = "PRODUCT_SN"),
        @Index(columnList = "CREATE_DATE_TIME"),
        @Index(columnList = "PART_NO"),
        @Index(columnList = "MODIFY_DATE_TIME"),
        @Index(columnList = "WORK_CENTER_CODE"),
        @Index(columnList = "WORK_CELL_CODE"),
        @Index(columnList = "WORK_ORDER_NO")
})
@Api("产品条码履历表")
public class MesProduceSnTravel extends BaseBean implements Serializable {
    private static final long serialVersionUID = -1807953720033107108L;

    @Column(name = "SERIAL_NUMBER")
    @ApiParam("过程条码")
    private String serialNumber;

    @Column(name = "PRODUCT_SN")
    @ApiParam("产品条码")
    private String productSn;

    @Column(name = "PART_NO")
    @ApiParam("物料号")
    private String partNo;

    @Column(name = "PART_NAME_RDD")
    @ApiParam("物料名称")
    private String partNameRdd;

    @Column(name = "QTY", columnDefinition = "decimal(18,8)")
    @ColumnDefault("0")
    @ApiParam("数量")
    private Double qty;

    @Column(name = "QC_STATUS")
    @ApiParam("质量状态")
    private Integer qcStatus;

    @Column(name = "SN_STATUS")
    @ApiParam("条码状态")
    private Integer snStatus;

    @Column(name = "PRINT_COUNT")
    @ApiParam("打印次数")
    private Integer printCount;

    @Column(name = "PRINT_STATUS")
    @ApiParam("打印状态")
    private Integer printStatus;

    @Column(name = "WORK_CENTER_CODE")
    @ApiParam("工作中心代码")
    private String workCenterCode;

    @Column(name = "WORK_CELL_CODE")
    @ApiParam("工作单元代码")
    private String workCellCode;

    @Column(name = "ROUTE_CODE")
    @ApiParam("流程代码")
    private String routeCode;

    @Column(name = "PROCESS_CODE")
    @ApiParam("工序代码")
    private String processCode;

    @Column(name = "NEXT_PROCESS_CODE")
    @ApiParam("下一工序代码")
    private String nextProcessCode;

    @Column(name = "IN_WC_TIME")
    @ApiParam("进产线时间")
    private String inWorkCenterTime;

    @Column(name = "OUT_WC_TIME")
    @ApiParam("出产线时间")
    private String outWorkCenterTime;

    @Column(name = "SHIPPING_TIME")
    @ApiParam("发运时间")
    private String shippingTime;

    @Column(name = "WORK_ORDER_NO")
    @ApiParam("生产工单号")
    private String workOrderNo;

    @Column(name = "CUST_SN")
    @ApiParam("客户条码")
    private String custSn;

    @Column(name = "CUST_CODE")
    @ApiParam("客户代码")
    private String custCode;

    @Column(name = "CUST_PART_NO")
    @ApiParam("客户零件号")
    private String custPartNo;

    @Column(name = "SN_TYPE")
    @ApiParam("条码类型 10=正常（默认），20=首检件")
    private Integer snType;

    @Column(name = "TRAY_NO")
    @ApiParam("托盘号")
    private String trayNo;

    @Column(name = "VIN_CODE")
    @ApiParam("Vin")
    private String vinCode;

    @Column(name = "OPERATE_TYPE")
    @ApiParam("操作类型")
    private Integer operateType;

    @Column(name = "DEFECT_ACTION_TYPE")
    @ApiParam("不良处理类型")
    private String defectActionType;

    @Column(name = "PROCESS_LABEL_TEMPLATE")
    @ApiParam("过程标签模板")
    private String processLabelTemplate;

    @Column(name = "PROD_LABEL_TEMPLATE")
    @ApiParam("产品标签模板")
    private String prodLabelTemplate;

    @Column(name = "CUST_LABEL_TEMPLATE")
    @ApiParam("客户标签模板")
    private String custLabelTemplate;

    @Column(name = "LOT_NO")
    @ApiParam("生产批次")
    private String lotNo;

    @Transient
    @ApiParam("返回信息")
    private String resultMsg;

    @Transient
    @ApiParam("下线开始时间")
    private String outWorkCenterTimeStart;

    @Transient
    @ApiParam("下线结束时间")
    private String outWorkCenterTimeEnd;

    @Transient
    @ApiParam("操作类型名称")
    private String operateTypeName;

    @Transient
    @ApiParam("条码状态名称")
    private String snStatusName;

    @Transient
    @ApiParam("质量状态名称")
    private String qcStatusName;

    @Transient
    @ApiParam("条码类型名称")
    private String snTypeName;

    public double getQtyVal() {
        return this.qty == null ? 0.0d : this.qty;
    }

    public int getQcStatusVal() {
        return this.qcStatus == null ? 0 : this.qcStatus;
    }

    public int getSnStatusVal() {
        return this.snStatus == null ? 0 : this.snStatus;
    }

    public int getPrintCountVal() {
        return this.printCount == null ? 0 : this.printCount;
    }

    public int getPrintStatusVal() {
        return this.printStatus == null ? 0 : this.printStatus;
    }
}
